/* --- Copyright University of Sussex 1999. All rights reserved. ----------
 > File:			C.win32/src/win32tchar.p
 > Purpose:			Win32: support for UNICODE APIs
 > Author:			Robert Duncan, Jan 24 1997 (see revisions)
 > Documentation:
 > Related Files:
 */

#_INCLUDE 'declare.ph'
#_INCLUDE 'win32defs.ph'


constant procedure ( $-Sys$-Get_string, $-Sys$-Try_encode_string, );

;;; -----------------------------------------------------------------------

section $-Sys;

	;;; Computes the size in bytes of a TSTRING of length _nchars
	;;; allowing room for the terminating null character
define :inline lconstant _TSTRING_SIZE(_nchars);
	(@@(b)[_nchars|TCHAR] _add --@@(b)[_1|TCHAR])
enddefine;

constant
	wkstring1  = writeable inits(_pint(_TSTRING_SIZE(_:WKSTRING_LENGTH))),
	wkstring2  = writeable copy(wkstring1),
	wkstring3  = writeable copy(wkstring1),
;

define Tchars_in(string, wkstring) -> wkstring;
#_IF DEF WIN32_UNICODE
	if string!KEY!K_FLAGS _bitst _:M_K_STRING16 then
		string -> wkstring;
	else
		;;; widen to 16 bits
		lvars _nchars = string!V_LENGTH;
		if _nchars _gr _:WKSTRING_LENGTH then
			Get_string16(_nchars) -> wkstring;
		endif;
		lvars _index = _0;
		while _index _lt _nchars do
			string!(b)[_index] -> wkstring!(s)[_index];
			_index _add _1 -> _index;
		endwhile;
		_0 -> wkstring!(s)[_nchars];
	endif;
#_ELSE
	lvars _encode = sys_encoding, _nbytes;
	if _encode or string!KEY!K_FLAGS _bitst _:M_K_STRING16 then
		if Try_encode_string(string, wkstring@V_BYTES, wkstring!V_LENGTH,
				_encode) -> _nbytes
		then
			_0 -> wkstring!(b)[_nbytes];
		else
			Get_string(_nbytes) -> wkstring;
			Try_encode_string(string, wkstring@V_BYTES, _nbytes, _encode)
					-> (_, _);
		endif;
	else
		string -> wkstring;
	endif;
#_ENDIF
enddefine;

define Tchars_out(wkstring, _nchars) -> string;
#_IF DEF WIN32_UNICODE
	lvars _index = _0, need16 = false;
	while _index _lt _nchars do
		if wkstring!(s)[_index] _gr _16:FF then
			true -> need16;
			quitloop;
		endif;
		_index _add _1 -> _index;
	endwhile;
	if need16 then
		Get_string16(_nchars) -> string;
		_smove(@@(s)[_nchars], wkstring@V_SHORTS[_0], string@V_SHORTS[_0]) -> ;
	else
		Get_string(_nchars) -> string;
		_0 -> _index;
		while _index _lt _nchars do
			wkstring!(s)[_index] -> string!(b)[_index];
			_index _add _1 -> _index;
		endwhile;
	endif;
#_ELSE
	lvars _decode = sys_encoding;
	if _decode then
		Consstring_bptr(wkstring@V_BYTES[_0], _nchars, CSB_POP_STRING, _decode)
				-> string;
	else
		Get_string(_nchars) -> string;
		_bmove(@@(b)[_nchars], wkstring@V_BYTES[_0], string@V_BYTES[_0]) -> ;
	endif;
#_ENDIF
enddefine;

endsection;		/* $-Sys */


/* --- Revision History ---------------------------------------------------
--- Robert Duncan, Oct 20 1999
		Modified non-Unicode cases to do explicit encode and decode
--- Robert Duncan, Feb 13 1997
		String16 changes
 */
